package com.chinaitop.depot.intelligent.api.controller;


import com.chinaitop.depot.intelligent.grainsituation.mapper.TTestdataLayerMapper;
import com.chinaitop.depot.intelligent.grainsituation.mapper.TWarningThresholdHistoryMapper;
import com.chinaitop.depot.intelligent.grainsituation.model.TTestdata;
import com.chinaitop.depot.intelligent.grainsituation.model.TTestdataLayer;
import com.chinaitop.depot.intelligent.grainsituation.model.TWarningThresholdHistory;
import com.chinaitop.depot.intelligent.grainsituation.service.TemperatureRecordService;
import com.chinaitop.depot.intelligent.utils.JsonToObjectUtils;
import com.unissoft.model.ResponseEntity;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;

/**
 * 文件描述
 *
 * @author tongxi.xia
 * @date 2019年12月26日 14:48
 */
@RestController
@RequestMapping(value = "/depotGrain")
public class GrainAPIController {

    @Resource
    private TemperatureRecordService temperatureRecordService;

    @Resource
    private TTestdataLayerMapper tTestdataLayerMapper;

    @Resource
    private TWarningThresholdHistoryMapper tWarningThresholdHistoryMapper;

    final static Logger logger = LoggerFactory.getLogger(GrainAPIController.class);

    /**
     * @MethodName: 接受终端测温数据
     * @Description: TODO
     * @Param:
     * @Return:
     * @Author: tongxi.xia
     * @Date: 2020/1/11
     **/
    @PostMapping("/acceptTemperatureData")
    public void acceptTemperatureData(@RequestBody String grainJson) {
        try {
            if (StringUtils.isNotBlank(grainJson)) {
                ArrayList<TTestdata> tTestdataClouds = new ArrayList<>();
                ArrayList<Object> objects = JsonToObjectUtils.jsonToObjectsDiverse(grainJson, TTestdata.class, "yyyy-MM-dd HH:mm:ss");
                objects.forEach(obj -> {
                    TTestdata s = (TTestdata) obj;
                    tTestdataClouds.add(s);
                });
                temperatureRecordService.insertBatch(tTestdataClouds);
            }
            logger.info("接受终端测温数据acceptTemperatureData保存成功：参数：grainJson=" + grainJson);
        } catch (Exception e) {
            logger.info("接受终端测温数据acceptTemperatureData保存失败：参数：grainJson=" + e.getMessage());
        }
    }

    @PostMapping("/acceptTemperatureDataLayer")
    public void acceptTemperatureDataLayer(@RequestBody String grainJson) {
        try {
            if (StringUtils.isNotBlank(grainJson)) {
                ArrayList<TTestdataLayer> tTestdataClouds = new ArrayList<>();
                ArrayList<Object> objects = JsonToObjectUtils.jsonToObjectsDiverse(grainJson, TTestdataLayer.class, "yyyy-MM-dd HH:mm:ss");
                objects.forEach(obj -> {
                    TTestdataLayer s = (TTestdataLayer) obj;
                    tTestdataClouds.add(s);
                });
                tTestdataLayerMapper.insertBatch(tTestdataClouds);
            }
            logger.info("接受终端测温数据acceptTemperatureDataLayer保存成功：参数：grainJson=" + grainJson);
        } catch (Exception e) {
            logger.info("接受终端测温数据acceptTemperatureDataLayer保存失败：参数：grainJson=" + e.getMessage());
        }
    }

    @PostMapping("/acceptTemperatureDataWarning")
    public void acceptTemperatureDataWarning(@RequestBody String grainJson) {
        try {
            if (StringUtils.isNotBlank(grainJson)) {
                ArrayList<TWarningThresholdHistory> tTestdataClouds = new ArrayList<>();
                ArrayList<Object> objects = JsonToObjectUtils.jsonToObjectsDiverse(grainJson, TWarningThresholdHistory.class, "yyyy-MM-dd HH:mm:ss");
                objects.forEach(obj -> {
                    TWarningThresholdHistory s = (TWarningThresholdHistory) obj;
                    tTestdataClouds.add(s);
                });
                tWarningThresholdHistoryMapper.insertBatch(tTestdataClouds);
            }
            logger.info("接受终端测温数据acceptTemperatureDataWarning保存成功：参数：grainJson=" + grainJson);
        } catch (Exception e) {
            logger.info("接受终端测温数据acceptTemperatureDataWarning保存失败：参数：grainJson=" + e.getMessage());
        }
    }

    @GetMapping("/test")
    public ResponseEntity test() {
        return ResponseEntity.ok("api测试成功");
    }
}
