package com.chinaitop.depot.intelligent.basicdata.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.intelligent.basicdata.mapper.TJctableMapper;
import com.chinaitop.depot.intelligent.basicdata.mapper.TJctableParmMapper;
import com.chinaitop.depot.intelligent.basicdata.model.TJctable;
import com.chinaitop.depot.intelligent.basicdata.model.TJctableExample;
import com.chinaitop.depot.intelligent.basicdata.model.TJctableParm;
import com.chinaitop.depot.intelligent.basicdata.service.QuantityService;
import com.chinaitop.depot.intelligent.basicdata.service.WareHouseBasicInfoService;
import com.chinaitop.depot.intelligent.utils.*;
import com.unissoft.model.BasicEntity;
import com.unissoft.model.SendEntity;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class QuantityServiceImpl implements QuantityService {
    final static Logger logger = LoggerFactory.getLogger(QuantityServiceImpl.class);
    @Value("${num_version3}")
    private String num_version3 = ":20013";
    @Resource
    private TJctableMapper tJctableMapper;
    @Resource
    private WareHouseBasicInfoService wareHouseBasicInfoService;
    @Resource
    private TJctableParmMapper parmMapper;


    @Override
    public List<TJctable> getList(String cfCode) {
        TJctableExample example = new TJctableExample();
        TJctableExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotEmpty(cfCode)) {
            criteria.andVCfCodeEqualTo(cfCode);
        }
        criteria.andDelFlagEqualTo(1);//未删除
        example.setOrderByClause("update_time DESC");
        return tJctableMapper.selectByExample(example);
    }

    @Override
    public TJctable getById(String id) {
        return tJctableMapper.selectByPrimaryKey(id);
    }

    @Override
    public TJctableParm getParamById(String id) {
        return parmMapper.selectByPrimaryKey(id);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void save(String grainJson, String grainBaseJson) {
        if (StringUtils.isNotBlank(grainJson) && StringUtils.isNotBlank(grainBaseJson)) {
            // 设备基础数据
            TJctable tJctable = (TJctable) JsonToObjectUtils.jsonToObject(grainJson, "com.chinaitop.depot.intelligent.basicdata.model.TJctable");
            tJctable.setUpdateTime(DateUtils.getDate());
            // 推送基础数据(致远)
            JSONObject jsonObject = SendEntity.synchronizationConvert(grainJson);

            // 库内粮食基础信息设置
            TJctableParm tJctableParm = JSONObject.parseObject(grainBaseJson, TJctableParm.class);
            tJctableParm.setCF(tJctable.getvCfCode()); // 设备设置的仓房编号
            // 对象转map
            Map<String, Object> map = BeanToMapUtil.convertBeanToMap(tJctableParm);

            // 同步到设备
            String sr;
            String code = UuidUtils.getCode(); // 生成主键uuid
            // 带数字的修改为字母key修改为大写 key=uuid&z0=1&s0=1&r0=1&s1=1
            map.put("key", code);
            map.put("Z0", map.remove("z0"));
            map.put("S0", map.remove("s0"));
            map.put("R0", map.remove("r0"));
            map.put("S1", map.remove("s1"));
            // 更新
            if (StringUtils.isNotBlank(tJctable.getId())) {
                // 设备表更新
                tJctableMapper.updateByPrimaryKeySelective(tJctable);
                // 移除更新人
                jsonObject.remove("updatePerson");
                // 移除不需要推送的参数
                jsonObject.remove("delFlag");
                jsonObject.remove("dataSource");
                jsonObject.remove("updateTime");
                // 推送数据并返回结果(致远)
                SendEntity.cxfFactor().saveJcTable(SendEntity.sendAndConvert(jsonObject, "U"));

                // 库内粮食基础信息更新
                tJctableParm.setKey(code); // 每次更新设备key_id,合工大只支持新增,不支持更新.
                // 发送设置数据
                sr = setBaseData(map, tJctable.getvWsSrc());
                // 是否设置到设备
                if (ifResult(sr)) {
                    tJctableParm.setgFlag(1); // 0:否,1:是
                }
                parmMapper.updateByPrimaryKeySelective(tJctableParm);
            } else { // 新增
                // 获取Max+1值
                Map<String, Object> valueMax = wareHouseBasicInfoService.getCode("t_jctable", "v_jc_code");
                // 替换值
                jsonObject.put("vJcCode", valueMax.get("column1"));
                tJctable.setId(code);
                tJctable.setDelFlag(1); //未删除
                tJctable.setvJcCode(valueMax.get("column1").toString());
                // 设备表新增
                tJctableMapper.insert(tJctable);
                // 推送数据并返回结果(致远)
                SendEntity.cxfFactor().saveJcTable(SendEntity.sendAndConvert(jsonObject, "I"));

                // 库内粮食基础信息更新
                tJctableParm.setId(code);
                tJctableParm.setKey(code);
                sr = setBaseData(map, tJctable.getvWsSrc());
                // 是否设置到设备
                if (ifResult(sr)) {
                    tJctableParm.setgFlag(1); // 0:否,1:是
                }
                parmMapper.insertSelective(tJctableParm);
            }
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int deleteById(String id, String quantityJson) {
        TJctable tJctable = (TJctable) JsonToObjectUtils.jsonToObject(quantityJson, "com.chinaitop.depot.intelligent.basicdata.model.TJctable");
        tJctable.setDelFlag(0); //删除
        int i = tJctableMapper.updateByPrimaryKeySelective(tJctable);
        // 先查询key_id
        String key = parmMapper.selectByPrimaryKey(id).getKey();
        // 远程删除设备粮情设置
        String s = delBaseData(key, tJctable.getvWsSrc());
        if (ifResult(s)) {
            // 删除设备粮情设置
            parmMapper.deleteByPrimaryKey(id);
        }
        // 推送基础数据
        if (i > 0) {
            JSONObject jsonObject = SendEntity.synchronizationConvert(quantityJson);
            // 移除不需要推送的参数
            jsonObject.remove("updatePerson");
            jsonObject.remove("dataSource");
            jsonObject.remove("updateTime");
            jsonObject.remove("delFlag");
            SendEntity.cxfFactor().saveJcTable(SendEntity.sendAndConvert(jsonObject, "D"));
        }
        return i;
    }

    @Override
    public boolean getBycfCode(String vdevcode, String vdevname) {
        TJctableExample example = new TJctableExample();
        TJctableExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotEmpty(vdevcode)) {
            criteria.andVJcCodeEqualTo(vdevcode);
        }
        if (StringUtils.isNotEmpty(vdevname)) {
            criteria.andVJcNameEqualTo(vdevname);
        }
        List<TJctable> tJctables = tJctableMapper.selectByExample(example);
        if (tJctables.size() > 0) {
            return true;
        }
        return false;
    }

    @Transactional
    @Override
    public void insertBatch(List<TJctable> tJctables) {
        for (TJctable t : tJctables) {
            List<TJctable> byCode = getByCode(t.getvJcCode());
            if (byCode.size() > 0) {
                tJctableMapper.updateByPrimaryKey(t);
            } else {
                tJctableMapper.insert(t);
            }
        }
//        tJctableMapper.insertBatch(tJctables);
    }

    @Override
    public List<TJctable> getByCodeAndTimes(List<String> cDcsCodes, List<Date> cTimes) {
        TJctableExample example = new TJctableExample();
        TJctableExample.Criteria criteria = example.createCriteria();
        if ((cDcsCodes != null && cDcsCodes.size() > 0) && (cTimes != null && cTimes.size() > 0)) {
            criteria.andVCfCodeIn(cDcsCodes);
            criteria.andUpdateTimeIn(cTimes);
            List<TJctable> tJctables = tJctableMapper.selectByExample(example);
            return tJctables;
        }
        return null;
    }

    @Override
    public List<TJctable> getByCode(String jcCode) {
        TJctableExample example = new TJctableExample();
        TJctableExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotBlank(jcCode)) {
            criteria.andVJcCodeEqualTo(jcCode);
            List<TJctable> tJctables = tJctableMapper.selectByExample(example);
            return tJctables;
        }
        return null;
    }

    @Transactional
    @Override
    public String synchronizationAll(String orgId) {
        //获取所有基础数据getJcTable
        String insectDevice = SendEntity.cxfFactor().getJcTable();
        //转换对象
        BasicEntity basicEntity = (BasicEntity) JsonToObjectUtils.jsonToObject(insectDevice, BasicEntity.class);
        if (basicEntity.getState().equals("0")) {
            List<Object> data = basicEntity.getData();
            if (data.size() > 0) {
                String tcs = JSONObject.toJSONString(data);
                ArrayList<Object> objects = JsonToObjectUtils.jsonToObjects(tcs, TJctable.class);
                objects.forEach(t -> {
                    TJctable tc = (TJctable) t;
                    tc.setId(UuidUtils.getCode());
                    tc.setOrgId(orgId);
                    tc.setDelFlag(1); //未删除
                    String vdevcode = tc.getvJcCode();
                    TJctable ccinfo1 = getCcinfo(vdevcode);
                    if (ccinfo1 != null) {
                        tc.setId(ccinfo1.getId());
                        tJctableMapper.updateByPrimaryKey(tc);
                    } else {
                        tJctableMapper.insert(tc);
                    }
                });
            }
        } else {
            return basicEntity.getMsg();
        }
        return basicEntity.getMsg();
    }

    @Override
    public List<TJctable> getByCodes(List<String> jcCodes) {
        TJctableExample example = new TJctableExample();
        TJctableExample.Criteria criteria = example.createCriteria();
        if (jcCodes.size() > 0) {
            criteria.andVRemarkIn(jcCodes);
        }
        return tJctableMapper.selectByExample(example);
    }

    public TJctable getCcinfo(String vdevcode) {
        TJctableExample example = new TJctableExample();
        TJctableExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotEmpty(vdevcode)) {
            criteria.andVJcCodeEqualTo(vdevcode);
        }
        List<TJctable> tCcinfos = tJctableMapper.selectByExample(example);
        if (tCcinfos.size() > 0) {
            return tCcinfos.get(0);
        }
        return null;
    }

    /**
     * 调用保存数量检测粮食参数配置service接口
     *
     * @param mapValue 参数
     * @param ws       硬件厂商接口地址
     * @return 硬件厂商返回调用信息
     */
    private String setBaseData(Map<String, Object> mapValue, String ws) {
        String msg = ""; // 初始化返回字符串(成功 suc 失败 unsuc_setBaseData_原因)
        try {
            // 判断参数map有值,并且硬件厂商webservice接口地址不能为空
            if (mapValue.size() > 0 && StringUtils.isNotBlank(ws)) {
                // 传给设备的仓房标识不能为空
                if (StringUtils.isNotBlank(mapValue.get("CF").toString())) {
                    // 获取配置文件配置硬件版本(:20013),默认(:20013),(:20012 弃用)
                    if (ws.contains(num_version3)) {
                        // 效验设备是否在线
                        if (!SoapConstant.getSafetyShebei(mapValue.get("CF").toString(), ws)) {
                            // 调用http get 请求方法 拼接请求url参数
                            msg = SoapConstant.setBaseDATA(mapValue, ws);
                            logger.info("setBaseData响应值为: {}", msg);
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return msg;
        }
        return msg;
    }

    private String delBaseData(String value, String ws) {
        logger.info("delBaseData值为: value:{},ws：{}", value, ws);
        String msg = ""; // 初始化返回字符串(成功 suc 失败 unsuc_setBaseData_原因)
        try {
            // 判断参数map有值,并且硬件厂商webservice接口地址不能为空
            if (StringUtils.isNotBlank(value) && StringUtils.isNotBlank(ws)) {
                // 获取配置文件配置硬件版本(:20013),默认(:20013),(:20012 弃用)
                if (ws.contains(num_version3)) {
                    // 效验设备是否在线
                    if (!SoapConstant.getSafetyShebei(value, ws)) {
                        // 河工大接口问题需要拼接 '' 才好使
                        value = "'" + value + "'";
                        logger.info("delBaseData拼接后的: value:{}", value);
                        // 调用http get 请求方法 拼接请求url参数
                        msg = SoapConstant.delBaseData(value, ws);
                        logger.info("delBaseData响应值为: {}", msg);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return msg;
    }

    /**
     * delBaseData: return 成功 suc_ delBaseData 失败 unsuc_ delBaseData _原因
     * setBaseData: return 成功 suc 失败 unsuc_setBaseData_原因
     *
     * @param strXml
     * @return
     */
    private boolean ifResult(String strXml) {
        boolean b = false;
        if (strXml.contains("suc")) {
            String s = StringXmlUtil.getStringXmlHead(strXml);
            logger.info("setBaseData返回解析值: {}", s);
            if ("suc".equals(s)) {
                b = true;
            }
        }
        return b;
    }

}
