package com.chinaitop.depot.intelligent.grainsituation.controller;

import com.chinaitop.depot.intelligent.basicdata.service.TUrlConfigService;
import com.chinaitop.depot.intelligent.grainsituation.model.TQtdevinfoValue;
import com.chinaitop.depot.intelligent.grainsituation.model.TTestdata;
import com.chinaitop.depot.intelligent.grainsituation.service.GasDetectionService;
import com.chinaitop.depot.intelligent.socket.SocketClient;
import com.chinaitop.depot.intelligent.utils.ConstUtils;
import com.chinaitop.depot.intelligent.utils.HexConvertUtils;
import com.chinaitop.depot.intelligent.utils.RemoteControlUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.unissoft.model.ResponseEntity;
import com.unissoft.model.TypeEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.*;

@RestController
@RequestMapping(value = "/intelligents/gasDetection")
@Api(value = "GasDetectionController", description = "气体检测记录控制类")
@SuppressWarnings("all")
public class GasDetectionController {
    @Resource
    private GasDetectionService gasDetectionService;
    @Resource
    private TUrlConfigService urlConfigService;
    @Resource
    private RemoteControlUtil remoteControlUtil;

    @RequestMapping(value = "/getList", method = RequestMethod.GET)
    @ApiOperation(value = "查询 气体 信息列表", notes = "查询 气体 信息列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "仓库名称", paramType = "query")
    })
    public ResponseEntity<PageInfo<Object>> getList(Integer pageNum, Integer pageSize, String vCfCode, String startTime, String endTime) {
        List<Object> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = gasDetectionService.getList(vCfCode, startTime, endTime, "");//查询方法添加id查询
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed(e.getMessage());
        }
        PageInfo<Object> pageInfo = new PageInfo<Object>(list);
        return ResponseEntity.ok(pageInfo);
    }

    @RequestMapping(value = "/getListGroup", method = RequestMethod.GET)
    @ApiOperation(value = "查询 气体 检测列表", notes = "查询 气体 检测列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "仓库名称", paramType = "query")
    })
    public ResponseEntity<PageInfo<Object>> getListGroup(Integer pageNum, Integer pageSize, String vCfCode) {
        List<Object> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = gasDetectionService.getListGroup(vCfCode);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed(e.getMessage());
        }
        PageInfo<Object> pageInfo = new PageInfo<Object>(list);
        return ResponseEntity.ok(pageInfo);
    }

    /**
     * orgid下的所有仓房粮情检测
     *
     * @param storeCode
     * @return
     */
    @Autowired
    private SocketClient socketClient;
    @Autowired
    private ConstUtils constUtils;

    /**
     * 单仓检测接口
     *
     * @param vCfCode
     * @return
     */
    @RequestMapping(value = "/sendOnlyGasDetection", method = RequestMethod.POST)
    @ApiOperation(value = "单仓气体实时检测", notes = "单仓气体实时检测")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "vCfCode", defaultValue = "001", value = "仓库编号", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "库点编号", paramType = "query")
    })
    public ResponseEntity sendOnlyInsectPestDetection(String vCfCode, String orgId) {
        String result = urlConfigService.getDepotUrl(orgId);
        if (StringUtils.isNotBlank(result)) {
            MultiValueMap<String, String> map = new LinkedMultiValueMap<>();
            map.add("vCfCode", vCfCode);
            map.add("orgId", orgId);
            result = remoteControlUtil.remoteScan(result + RemoteControlUtil.gasOnly, map, "POST", null);
        } else {
            result = TypeEnum.S_STX.getDesc();
        }
        return ResponseEntity.ok(result);
    }

    /**
     * 所有仓实时检测
     *
     * @return
     */
    @RequestMapping(value = "/sendGasDetectionAll", method = RequestMethod.GET)
    @ApiOperation(value = "所有仓气体实时检测", notes = "所有仓气体实时检测")
    @ApiImplicitParam(name = "orgId", value = "库点编号", paramType = "query")
    public ResponseEntity sendTemperatureDetectionAll(@NotNull String orgId) {
        String result = urlConfigService.getDepotUrl(orgId);
        if (StringUtils.isNotBlank(result)) {
            Map<String, String> map = new HashMap<>();
            map.put("orgId", orgId);
            result = remoteControlUtil.remoteScan(result + RemoteControlUtil.gasAll, null, "GET", map);
        } else {
            result = TypeEnum.S_STX.getDesc();
        }
        return ResponseEntity.ok(result);
    }

}
