package com.chinaitop.depot.intelligent.grainsituation.controller;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.intelligent.basicdata.service.TUrlConfigService;
import com.chinaitop.depot.intelligent.common.ResultEnum;
import com.chinaitop.depot.intelligent.grainsituation.model.TTestdata;
import com.chinaitop.depot.intelligent.grainsituation.model.TTestdataLayer;
import com.chinaitop.depot.intelligent.grainsituation.model.TempMetadata;
import com.chinaitop.depot.intelligent.grainsituation.service.TPointsService;
import com.chinaitop.depot.intelligent.grainsituation.service.TemperatureRecordService;
import com.chinaitop.depot.intelligent.socket.SocketClient;
import com.chinaitop.depot.intelligent.utils.ConstUtils;
import com.chinaitop.depot.intelligent.utils.HexConvertUtils;
import com.chinaitop.depot.intelligent.utils.ProgressBarUtil;
import com.chinaitop.depot.intelligent.utils.RemoteControlUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.google.gson.reflect.TypeToken;
import com.unissoft.model.ResponseEntity;
import com.unissoft.model.TypeEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping(value = "/intelligents/temperatureRecord")
@Api(value = "TemperatureRecordController", description = "测温记录控制类")
@SuppressWarnings("all")
public class TemperatureRecordController {

    @Autowired
    private TemperatureRecordService temperatureRecordService;
    @Autowired
    private TPointsService tpointsService;
    @Autowired
    private TUrlConfigService tUrlConfigService;
    @Autowired
    private RestTemplate restTemplate;
    @Resource
    private RemoteControlUtil remoteControlUtil;

    @RequestMapping(value = "/getList", method = RequestMethod.GET)
    @ApiOperation(value = "查询 测温 信息列表", notes = "查询 测温 信息列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "vDatatime", value = "采集时间", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "仓库名称", paramType = "query"),
            @ApiImplicitParam(name = "tType", value = "检测类型0测温，1粮油,2测水分", paramType = "query")
    })
    public ResponseEntity<PageInfo<TTestdata>> getList(Integer pageNum, Integer pageSize, String vCfCode, String vDatatime, String startTime, String endTime, String tType, String orgId) {
        List<TTestdata> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = temperatureRecordService.getList(vCfCode, vDatatime, startTime, endTime, tType, orgId);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed(e.getMessage());
        }
        PageInfo<TTestdata> pageInfo = new PageInfo<TTestdata>(list);
        return ResponseEntity.ok(pageInfo);
    }

    // 要求修改为->温湿度检测只显示当天数据,按仓房排序
    @RequestMapping(value = "/getListOfGroup", method = RequestMethod.GET)
    @ApiOperation(value = "查询 测温 信息列表", notes = "查询 测温 信息列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "vDatatime", value = "采集时间", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "仓库名称", paramType = "query"),
            @ApiImplicitParam(name = "tType", value = "检测类型0测温，1粮油,2测水分", paramType = "query")
    })
    public ResponseEntity getListOfGroup(Integer pageNum, Integer pageSize, String vCfCode, String vDatatime, String tType, String orgId) {
        List<TTestdata> list = null;
        try {
            // 查询检测类型不能为空(检测类型0测温，1粮油,2测水分)
            if(StringUtils.isBlank(tType))
                return ResponseEntity.failed(ResultEnum.CODE_20.getMsg());
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = temperatureRecordService.getListOfGroup(vCfCode, vDatatime, tType, orgId);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed(e.getMessage());
        }
        PageInfo<TTestdata> pageInfo = new PageInfo<TTestdata>(list);
        return ResponseEntity.ok(pageInfo);
    }

    // 要求修改为->分页列表数据(测温历史记录列表)
    @RequestMapping(value = "/getTemHistoryPageInfoList", method = RequestMethod.GET)
    @ApiOperation(value = "查询 测温 信息列表", notes = "查询 测温 信息列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "vDatatime", value = "采集时间", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "仓库名称", paramType = "query"),
            @ApiImplicitParam(name = "tType", value = "检测类型0测温，1粮油,2测水分", paramType = "query")
    })
    public ResponseEntity getTemHistoryPageInfoList(Integer pageNum, Integer pageSize, String vCfCode, String vDatatime, String startTime, String endTime, String tType, String orgId) {
        List<TTestdata> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = temperatureRecordService.getTemHistoryPageInfoList(vCfCode, vDatatime, startTime, endTime, tType, orgId);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed(e.getMessage());
        }
        PageInfo<TTestdata> pageInfo = new PageInfo<TTestdata>(list);
        return ResponseEntity.ok(pageInfo);
    }

    /**
     * 获取一条数据
     *
     * @param vCfCode   库房编码
     * @param vDatatime 检测时间
     * @return
     */
    @RequestMapping(value = "/findByHouseAndTime", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据库房编码和测温时间查询单条检测记录包括每层的最高温最低温平均温", notes = "根据库房编码和测温时间查询单条检测记录包括每层的最高温最低温平均温")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "lqId", value = "粮情id", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "库房编码", paramType = "query"),
            @ApiImplicitParam(name = "vDatatime", value = "测温时间", paramType = "query")
    })
    public ResponseEntity findByHouseAndTime(String lqId, String vCfCode, String vDatatime) {
        List<TTestdataLayer> tTestdataLayer = null;
        try {
            tTestdataLayer = temperatureRecordService.findByHouseAndTime(lqId, vCfCode, vDatatime);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed("查询失败");
        }
        return ResponseEntity.ok(tTestdataLayer);
    }

    /**
     * @param lqId
     * @param vCfCode
     * @param vDatatime
     * @param viewFlag  true: 表格数据,false: 3D三温图数据
     * @return
     */
    @RequestMapping(value = "/findByHouseAndTimes", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据库房编码和测温时间查询检测记录包括层，行，各个节点的值", notes = "根据库房编码和测温时间查询检测记录包括层，行，各个节点的值")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "lqId", value = "粮情id", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "库房编码", paramType = "query"),
            @ApiImplicitParam(name = "vDatatime", value = "测温时间", paramType = "query"),
            @ApiImplicitParam(name = "viewFlag", value = "区分表格和3D图", paramType = "query")
    })
    public ResponseEntity findByHouseAndTimes(String lqId, String vCfCode, String vDatatime, boolean viewFlag) {
        ArrayList<Object> byHouseAndTimes = null;
        try {
            byHouseAndTimes = temperatureRecordService.findByHouseAndTimes(lqId, vCfCode, vDatatime, viewFlag);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed("查询失败");
        }
        return ResponseEntity.ok(byHouseAndTimes);
    }

    /**
     * 根据id获取带标杆号温度表格
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/findByTempTable", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "根据id查询粮情点温,带标杆号.", notes = "根据id查询检测记录包括层，行，列各个节点的值")
    @ApiImplicitParam(name = "id", value = "粮情id", paramType = "query")
    public ResponseEntity findByTempTable(String id) {
        Map<Integer, List<TempMetadata>> tempTable = null;
        try {
            tempTable = temperatureRecordService.findByTempTable(id);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed("查询失败");
        }
        return ResponseEntity.ok(tempTable);
    }

    /**
     * 左图表对比接口
     *
     * @param pageNum
     * @param pageSize
     * @param vCfCodeLeft
     * @param startTimeLeft
     * @param endTimeLeft
     * @return
     */
    @RequestMapping(value = "/getComparisonLeft", method = RequestMethod.GET)
    @ApiOperation(value = "左图表对比信息列表", notes = "左图表对比列表,支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", defaultValue = "1", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", defaultValue = "7", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "vCfCodeLeft", value = "左图仓房编码", paramType = "query"),
            @ApiImplicitParam(name = "startTimeLeft", value = "左图开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTimeLeft", value = "左图结束时间", paramType = "query"),
            @ApiImplicitParam(name = "tType", value = "检测类型0测温，1粮油,2测水分", paramType = "query")
    })
    public ResponseEntity<PageInfo<TTestdata>> getComparisonLeft(Integer pageNum, Integer pageSize, String vCfCodeLeft,
                                                                 String startTimeLeft, String endTimeLeft, String tType) {
        List<TTestdata> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = temperatureRecordService.getComparison(vCfCodeLeft, startTimeLeft, endTimeLeft, null, null, null, tType);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed(e.getMessage());
        }
        PageInfo<TTestdata> pageInfo = new PageInfo<TTestdata>(list);
        return ResponseEntity.ok(pageInfo);
    }

    /**
     * 左图表对比接口
     *
     * @param pageNum
     * @param pageSize
     * @param vCfCodeRight
     * @param startTimeRight
     * @param endTimeRight
     * @return
     */
    @RequestMapping(value = "/getComparisonRight", method = RequestMethod.GET)
    @ApiOperation(value = "右图表对比信息列表", notes = "右图表对比列表,支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", defaultValue = "1", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", defaultValue = "7", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "vCfCodeRight", value = "右图仓房编码", paramType = "query"),
            @ApiImplicitParam(name = "startTimeRight", value = "右图开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTimeRight", value = "右图结束时间", paramType = "query"),
            @ApiImplicitParam(name = "tType", value = "检测类型0测温，1粮油,2测水分", paramType = "query")
    })
    public ResponseEntity<PageInfo<TTestdata>> getComparisonRight(Integer pageNum, Integer pageSize, String vCfCodeRight,
                                                                  String startTimeRight, String endTimeRight, String tType) {
        List<TTestdata> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = temperatureRecordService.getComparison(null, null, null, vCfCodeRight, startTimeRight, endTimeRight, tType);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed(e.getMessage());
        }
        PageInfo<TTestdata> pageInfo = new PageInfo<TTestdata>(list);
        return ResponseEntity.ok(pageInfo);
    }

    /**
     * 所有点粮情对比
     *
     * @param pageNum
     * @param pageSize
     * @param storehouse
     * @param time
     * @return
     */
    @RequestMapping(value = "/getGrainPointAll", method = RequestMethod.GET)
    @ApiOperation(value = "点粮情对比", notes = "点粮情对比，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", defaultValue = "1", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", defaultValue = "30", paramType = "query"),
            @ApiImplicitParam(name = "time", value = "采集时间", paramType = "query"),
            @ApiImplicitParam(name = "storehouse", value = "仓库编号", paramType = "query"),
            @ApiImplicitParam(name = "id", value = "粮情,水分id", paramType = "query")
    })
    public ResponseEntity<PageInfo<Map<String, Object>>> getGrainPointAll(Integer pageNum, Integer pageSize, String storehouse, String time, String id) {
        List<Map<String, Object>> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = tpointsService.getGrainPointAll(storehouse, time, id);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed(e.getMessage());
        }
        PageInfo<Map<String, Object>> pageInfo = new PageInfo<Map<String, Object>>(list);
        return ResponseEntity.ok(pageInfo);
    }

    /**
     * orgid下的所有仓房粮情检测
     *
     * @param storeCode
     * @return
     */
    @Autowired
    private SocketClient socketClient;
    @Autowired
    private ConstUtils constUtils;

    @RequestMapping(value = "/sendTemperatureDetection", method = RequestMethod.POST)
    @ApiOperation(value = "粮情实时检测", notes = "粮情实时检测")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "storeCode", defaultValue = "001", value = "仓库编号", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "机构编码", paramType = "query")
    })
    public ResponseEntity sendTemperatureDetection(String storeCode, @NotNull String orgId) {
        Map<String, String> map = new LinkedHashMap<>();
        try {
            if (StringUtils.isNotBlank(storeCode)) {
                String depoturl = tUrlConfigService.getDepotUrl(orgId);
                if (StringUtils.isNotBlank(depoturl)) {
                    MultiValueMap multiValueMap = new LinkedMultiValueMap();
                    multiValueMap.add("storeCode", storeCode);
                    String result = remoteControlUtil.remoteScan(depoturl + RemoteControlUtil.chooseCFCode, multiValueMap, "POST", null);
                    if (result.contains("=")) {
                        Gson gson = new Gson();
                        map = gson.fromJson(result, map.getClass());
                        return ResponseEntity.ok(map);
                    } else {
                        return ResponseEntity.ok(result);
                    }
                } else {
                    return ResponseEntity.ok(TypeEnum.S_STX.getDesc());
                }
            } else {
                return ResponseEntity.ok(TypeEnum.S_EM.getDesc());  // 仓房号为空
            }
        } catch (JsonSyntaxException e) {
            e.printStackTrace();
            return ResponseEntity.ok(TypeEnum.S_STX.getDesc());
        }
    }

    @RequestMapping(value = "/sendTemperatureDetectionAll", method = RequestMethod.POST)
    @ApiOperation(value = "所有仓粮情实时检测", notes = "所有仓粮情实时检测")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "库点编码", paramType = "query"),
            @ApiImplicitParam(name = "tType", value = "检测类型0测温，1粮油,2测水分", paramType = "query")
    })
    public ResponseEntity sendTemperatureDetectionAll(@NotNull String orgId, @NotNull String tType) {
        try {
            String depoturl = tUrlConfigService.getDepotUrl(orgId);
            if (StringUtils.isNotBlank(depoturl)) {
                MultiValueMap<String, String> requestEntity = new LinkedMultiValueMap<>();
                requestEntity.add("tType", tType);
                String result = remoteControlUtil.remoteScan(depoturl + RemoteControlUtil.foodAll, requestEntity, "POST", null);
                return ResponseEntity.ok(result);
            } else {
                return ResponseEntity.ok(TypeEnum.S_STX.getDesc());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.ok(TypeEnum.S_STX.getDesc());
        }
    }

    /**
     * 单仓检测接口
     *
     * @param storeCode
     * @return
     */
    @RequestMapping(value = "/sendOnlyStoreDetection", method = RequestMethod.POST)
    @ApiOperation(value = "单仓粮情实时检测", notes = "单仓粮情实时检测")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "库点编码", paramType = "query"),
            @ApiImplicitParam(name = "storeCode", defaultValue = "001", value = "仓库编号", paramType = "query"),
            @ApiImplicitParam(name = "tType", value = "检测类型0测温，1粮油,2测水分", paramType = "query")
    })
    public ResponseEntity sendOnlyStoreDetection(@NotNull String orgId, @NotNull String storeCode, @NotNull String tType) {
        try {
            String depoturl = tUrlConfigService.getDepotUrl(orgId);
            if (StringUtils.isNotBlank(depoturl)) {
                MultiValueMap<String, String> requestEntity = new LinkedMultiValueMap<>();
                requestEntity.add("storeCode", storeCode);
                requestEntity.add("tType", tType);
                String result = remoteControlUtil.remoteScan(depoturl + RemoteControlUtil.foodOnly, requestEntity, "POST", null);
                return ResponseEntity.ok(result);
            } else {
                return ResponseEntity.ok(TypeEnum.S_STX.getDesc());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.ok(TypeEnum.S_STX.getDesc());
        }
    }

    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    @ApiOperation(value = "根据粮情id查询粮情信息", notes = "根据粮情id查询粮情信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "粮情id", paramType = "query")
    })
    public ResponseEntity findById(String id) {
        TTestdata byId = null;
        try {
            byId = temperatureRecordService.getById(id);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed("查询失败");
        }
        return ResponseEntity.ok(byId);
    }

    @Resource
    private ProgressBarUtil progressBarUtil;

    /**
     * @param detType  为'0'表示用户使用的全仓检测
     *                 为'1'表示用户使用的手动选仓检测
     *                 为'2'表示用户使用的单仓检测
     *                 为'3'表示页面定时刷新,需查询是否有正在执行的检测任务
     * @param cfNum    检测的仓房 全仓检测为空''
     * @param devState 检测设备是否连接成功
     * @return progressBarObj={"showProgress":"false","progress":0}
     */
    @RequestMapping(value = "/queryProgressBar", method = RequestMethod.POST)
    @ApiOperation(value = "进度条接口", notes = "根据检测生成粮情检测进度")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "detType", value = "粮情检测类型", paramType = "query"),
            @ApiImplicitParam(name = "cfNum", value = "粮情仓房code", paramType = "query"),
            @ApiImplicitParam(name = "devState", value = "粮情检测是否连接成功", paramType = "query")
    })
    public JSONObject queryProgressBar(String detType, String cfNum, String devState, String orgId) {
        // 返回json对象
        JSONObject progressBarObj = new JSONObject();
        try {
            // 1.判断检测是否连接成功,必须为(检测开始!)对应编码为(S_SOH)
            if (TypeEnum.S_SOH.getDesc().equals(devState)) {
                // 判断检测类型(全仓,手动选仓,单仓,定时刷新)
                progressBarObj.put("counter", progressBarUtil.typeIf(detType, cfNum, orgId));
                progressBarObj.put("showProgress", false);
            } else {
                // 连接不成功返回
                progressBarObj.put("showProgress", true);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return progressBarObj;
    }

    /**
     * 根据粮情id获取最高最低温部位
     * @param id
     * @return
     */
    @RequestMapping(value = "/getMaxAndMinDot", method = RequestMethod.GET)
    @ApiOperation(value = "根据粮情id获取最高最低温部位", notes = "根据粮情id获取最高最低温部位")
    @ApiImplicitParam(name = "id", value = "粮情id", paramType = "query")
    public ResponseEntity getMaxAndMinDot(String id) {
        Map<String,Object> resultMap = temperatureRecordService.getMaxAndMinDot(id);
        return ResponseEntity.ok(resultMap);
    }


}
