package com.chinaitop.depot.intelligent.levelDetection.controller;

import com.chinaitop.depot.intelligent.basicdata.service.TUrlConfigService;
import com.chinaitop.depot.intelligent.levelDetection.model.TLevel;
import com.chinaitop.depot.intelligent.levelDetection.service.LevelService;
import com.chinaitop.depot.intelligent.utils.RemoteControlUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.unissoft.model.ResponseEntity;
import com.unissoft.model.TypeEnum;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/intelligents/LevelDetection")
@Api(value = "LevelController", description = "液位记录控制类")
@SuppressWarnings("all")
public class LevelController {

    @Autowired
    private LevelService levelService;
    @Resource
    private TUrlConfigService urlConfigService;
    @Resource
    private RemoteControlUtil remoteControlUtil;

    @RequestMapping(value = "/getList", method = RequestMethod.GET)
    @ApiOperation(value = "查询 液位 信息列表", notes = "查询 液位 信息列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "vDatatime", value = "采集时间", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "仓库名称", paramType = "query")
    })
    public ResponseEntity<PageInfo<TLevel>> getList(Integer pageNum, Integer pageSize, String vCfCode, String vDatatime, String startTime, String endTime, String orgId) {
        List<TLevel> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = levelService.getList(vCfCode, vDatatime, startTime, endTime, orgId);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed(e.getMessage());
        }
        PageInfo<TLevel> pageInfo = new PageInfo<TLevel>(list);
        return ResponseEntity.ok(pageInfo);
    }

    /**
     * 液位时实数据查询,只查询最新的一次检测数据
     * @param pageNum
     * @param pageSize
     * @param vCfCode
     * @param orgId 拦截器提供
     * @return
     */
    @RequestMapping(value = "/getLevelRealPageInfo", method = RequestMethod.GET)
    @ApiOperation(value = "查询 液位 信息列表", notes = "查询 液位 信息列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "vCfCode", value = "仓库名称", paramType = "query")
    })
    public ResponseEntity getLevelRealPageInfo(Integer pageNum, Integer pageSize, String vCfCode) {
        List<TLevel> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = levelService.getLevelRealPageInfo(vCfCode);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed(e.getMessage());
        }
        PageInfo<TLevel> pageInfo = new PageInfo<TLevel>(list);
        return ResponseEntity.ok(pageInfo);
    }

    /**
     * orgid下的所有仓房粮情检测
     *
     * @param storeCode
     * @return
     */
    @RequestMapping(value = "/sendLevelDetectionAll", method = RequestMethod.GET)
    @ApiOperation(value = "所有液位实时检测", notes = "所有液位实时检测")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "tType", value = "液位", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织编码", paramType = "query")
    })
    public ResponseEntity sendLevelDetectionAll(String tType, @NotNull String orgId) {
        String result = urlConfigService.getDepotUrl(orgId);
        if (StringUtils.isNotBlank(result)) {
            Map<String, String> map = new HashMap<>();
            map.put("tType", tType);
            result = remoteControlUtil.remoteScan(result + RemoteControlUtil.levelAll, null, "GET", map);
        } else {
            result = TypeEnum.S_STX.getDesc();
        }
        return ResponseEntity.ok(result);
    }

    /**
     * 单仓检测接口
     *
     * @param storeCode 油罐编码
     * @return
     */
    @RequestMapping(value = "/sendOnlyStoreDetection", method = RequestMethod.POST)
    @ApiOperation(value = "单仓液位实时检测", notes = "单仓液位实时检测")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "storeCode", defaultValue = "YG001", value = "油罐编码", paramType = "query"),
            @ApiImplicitParam(name = "tType", value = "液位检测", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织编码", paramType = "query")
    })
    public ResponseEntity sendOnlyStoreDetection(String storeCode, String tType, @NotNull String orgId) {
        String result = urlConfigService.getDepotUrl(orgId);
        if (StringUtils.isNotBlank(result)) {
            MultiValueMap<String, String> map = new LinkedMultiValueMap<>();
            map.add("storeCode", storeCode);
            map.add("tType", tType);
            result = remoteControlUtil.remoteScan(result + RemoteControlUtil.onlyLevel, map, "POST", null);
        } else {
            result = TypeEnum.S_STX.getDesc();
        }
        return ResponseEntity.ok(result);
    }
    
    @RequestMapping(value = "/getById", method = RequestMethod.GET)
    @ApiOperation(value = "根据液位数据id查询 信息", notes = "根据液位数据id查询 信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "气体id", paramType = "query")
    })
    public ResponseEntity<TLevel> getById(String id) {
    	TLevel level = null;
        if (StringUtils.isNotBlank(id)) {
        	level = levelService.getById(id);
        } else {
            return ResponseEntity.failed("ID数据有误!");
        }
        return ResponseEntity.ok(level);
    }
    
    @RequestMapping(value = "/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "数据更新与添加", notes = "数据更新与添加")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "grainJson", value = "表数据json串", paramType = "form")
    })
    public ResponseEntity save(String grainJson) {
        try {
        	levelService.save(grainJson);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.failed("保存失败");
        }
        return ResponseEntity.ok();
    }
}
