package com.chinaitop.depot.intelligent.levelDetection.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.intelligent.basicdata.service.WareHouseBasicInfoService;
import com.chinaitop.depot.intelligent.levelDetection.mapper.TLevelMapper;
import com.chinaitop.depot.intelligent.levelDetection.model.TLevel;
import com.chinaitop.depot.intelligent.levelDetection.model.TLevelExample;
import com.chinaitop.depot.intelligent.levelDetection.service.LevelService;
import com.chinaitop.depot.intelligent.utils.DateUtils;
import com.chinaitop.depot.intelligent.utils.JsonToObjectUtils;
import com.chinaitop.depot.intelligent.utils.UuidUtils;
import com.chinaitop.depot.intelligent.weather.model.TWeatherhsitory;
import com.chinaitop.depot.intelligent.weather.model.TWertherinfo;
import com.unissoft.model.SendEntity;

import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import java.util.Date;
import java.util.List;
import java.util.Map;

@SuppressWarnings("all")
@Service
public class LevelServiceImpl implements LevelService {
    @Resource
    private TLevelMapper levelMapper;
    
    @Resource
    private WareHouseBasicInfoService wareHouseBasicInfoService;
    @Resource
    private UuidUtils uUidUtils;

    @Override
    public List<TLevel> getList(String vCfCode, String vDatatime, String startTime, String endTime, String orgId) {
        TLevelExample example = new TLevelExample();
        example.setDistinct(true);
        TLevelExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotEmpty(vCfCode)) {
            criteria.andOiltankCodeEqualTo(vCfCode);
        }
        if (StringUtils.isNotEmpty(vDatatime)) {
            criteria.andTTimeBetween(DateUtils.stringToData(vDatatime + DateUtils.startTime), DateUtils.stringToData(vDatatime + DateUtils.endTime));
        }
        if (StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime)) {
            criteria.andTTimeBetween(DateUtils.stringToData(startTime + DateUtils.startTime), DateUtils.stringToData(endTime + DateUtils.endTime));
        }
        if (StringUtils.isNotEmpty(orgId)) {
            criteria.andOrgIdEqualTo(orgId);
        }
        example.setOrderByClause("t_time desc");
        return levelMapper.selectByExample(example);
    }

	@Override
	public TLevel getById(String id) {
		return levelMapper.selectByPrimaryKey(id);
	}

	@Override
	public void save(String levelJson) {
		if (StringUtils.isNotBlank(levelJson)) {
            TLevel tlevel = (TLevel) JsonToObjectUtils.jsonToObject(levelJson, TLevel.class);
            tlevel.setUpdatetime(DateUtils.getDate());
            // 推送基础数据-转json对象过滤属性
            JSONObject jsonObject = SendEntity.synchronizationConvert(levelJson);
            // 更新时间
            jsonObject.put("updatetime",DateUtils.dateToString(tlevel.getUpdatetime()));
            if (StringUtils.isNotBlank(tlevel.getId())) {
            	levelMapper.updateByPrimaryKey(tlevel);
                // 推送数据并返回结果
                SendEntity.cxfFactor().saveGasDevice(SendEntity.sendAndConvert(jsonObject, "U"));
            } else {
                // 获取Max+1值
                Map<String, Object> valueMax = wareHouseBasicInfoService.getCode("t_wertherinfo", "sitecode", "vqxzcode", null);
                // 替换值
                jsonObject.put("vqxzcode",valueMax.get("column2"));
                // 保存数据
                tlevel.setId(uUidUtils.getCode());
//                tlevel.setVqxzcode(valueMax.get("column2").toString());
//                tlevel.setDelFlag(1); //未删除
                levelMapper.insert(tlevel);
                // 推送数据并返回结果
                SendEntity.cxfFactor().saveWeatherDevice(SendEntity.sendAndConvert(jsonObject, "I"));
            }
        }
	}

    /**
     * 液位时实数据查询,只查询最新的一次检测数据
     * @param vCfCode
     * @return
     */
    @Override
    public List<TLevel> getLevelRealPageInfo(String vCfCode) {
        return levelMapper.getLevelRealPageInfo(vCfCode);
    }

	@Override
	public int deleteById(String id, String leveljson) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public boolean getBycfCode(String vdevcode, String vdevname) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public List<TWeatherhsitory> getByCodeAndTimes(List<String> cDcsCodes, List<Date> cTimes) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void insertBatch(List<TWeatherhsitory> tEnergyInfos) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public List<TLevel> getByCode(String cDcsCode) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String synchronizationAll(String orgId) {
		// TODO Auto-generated method stub
		return null;
	}

}
