package com.chinaitop.depot.intelligent.utils;

import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;

/**
 * @author qingsong.han
 * @description: http工具类
 * @create 2020-04-19 12:48
 */
@SuppressWarnings("all")
public class HttpUtil {
    private final static Logger log = LoggerFactory.getLogger(HttpUtil.class);

    /**
     * 使用apache的HttpClient发送http
     *
     * @param wsdlURL     请求URL
     * @param contentType 如:application/json;charset=utf8
     * @param content     数据内容
     * @DATE 2018年9月22日 下午10:29:17
     */
    public static String doHttpGetByHttpClient(final String wsdlURL, final String contentType, final String content) throws IOException {
        // 获得Http客户端(可以理解为:你得先有一个浏览器;注意:实际上HttpClient与浏览器是不一样的)
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        HttpGet httpGet = new HttpGet(wsdlURL);
        // 响应模型
        CloseableHttpResponse response = null;
        String result = "";
        try {
            response = httpClient.execute(httpGet);
            /*
             * 从响应模型中获取响应实体
             * 注意:和doHttpPostByRestTemplate方法用的不是同一个HttpEntity
             */
            HttpEntity responseEntity = response.getEntity();
            if (responseEntity != null) {
                result = EntityUtils.toString(responseEntity);
                log.info("响应ContentType为: {}", responseEntity.getContentType());
                log.info("响应状态为: {}", response.getStatusLine());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
        } finally {
            // 释放资源
            if (httpClient != null) {
                httpClient.close();
            }
            if (response != null) {
                response.close();
            }
        }
        log.info("响应内容为: {}", result);
        return result;
    }

    /*public static String doHttpPostByHttpClient(final String wsdlURL, final String contentType, final String content) throws IOException {
        // 获得Http客户端(可以理解为:你得先有一个浏览器;注意:实际上HttpClient与浏览器是不一样的)
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        // 创建Post请求
//        HttpPost httpPost = new HttpPost(wsdlURL);
        HttpGet httpGet = new HttpGet(wsdlURL);
        *//*StringEntity entity = new StringEntity(content, "UTF-8");
        // 将数据放入entity中
        httpPost.setEntity(entity);
        httpPost.setHeader("Content-Type", contentType);*//*
        // 响应模型
        CloseableHttpResponse response = null;
        String result = "";
        try {
            // 由客户端执行(发送)Post请求
//            response = httpClient.execute(httpPost);
            response = httpClient.execute(httpGet);
            // 从响应模型中获取响应实体
            // 注意:和doHttpPostByRestTemplate方法用的不是同一个HttpEntity
            org.apache.http.HttpEntity responseEntity = response.getEntity();
            System.out.println("响应ContentType为:" + responseEntity.getContentType());
            System.out.println("响应状态为:" + response.getStatusLine());
            if (responseEntity != null) {
                result = EntityUtils.toString(responseEntity);
                System.out.println("响应内容为:" + result);
            }
        } finally {
            // 释放资源
            if (httpClient != null) {
                httpClient.close();
            }
            if (response != null) {
                response.close();
            }
        }
        return result;
    }*/

    /**
     * 使用springframework的RestTemplate发送http
     * http使用无参构造;https需要使用有参构造
     *
     * @param wsdlURL     请求URL
     * @param contentType 如: application/x-www-form-urlencoded
     * @param content     数据内容
     *                    Content-Length: length
     *                    key=string&CF=string&ZBS=string&DELV=string&RKDate=string&CCLX=string&DZFS=string&XZ=string&R0=string&S0=string&S1=string&Z0=string&BWSLL=string&LSPZ=string&TFCS=string&TSQK=string
     * @DATE 2018年9月22日 下午10:30:48
     */
    public static String doHttpPostByRestTemplate(final String wsdlURL, final String contentType, final String content) {
        RestTemplate restTemplate = new RestTemplate();
        // 设置Content-Type
        HttpHeaders headers = new HttpHeaders();
        headers.add("Content-Type", contentType);
        // 注意:和doHttpPostByHttpClient方法用的不是同一个HttpEntity
        String result = restTemplate.postForObject(wsdlURL, content, String.class);
        return result;
    }

    /**
     * 封装的get请求，暂时只支持map传参，并且value只支持基本类型和String
     *
     * @param url    库端配置url
     * @param object 接收数据(只支持map类型)
     * @return string类型带参数get请求url
     */
    public static String getForObject(String url, Object object) {
        StringBuilder stringBuilder = new StringBuilder(url);
        if (object instanceof Map) {
            Iterator iterator = ((Map) object).entrySet().iterator();
            if (iterator.hasNext()) {
                stringBuilder.append("?");
                Object element;
                while (iterator.hasNext()) {
                    element = iterator.next();
                    Map.Entry entry = (Map.Entry) element;
                    //过滤value为null，value为null时进行拼接字符串会变成 "null"字符串
                    if (entry.getValue() != null) {
                        stringBuilder.append(element).append("&");
                    }
                    url = stringBuilder.substring(0, stringBuilder.length() - 1);
                }
            }
        } else {
            throw new RuntimeException("url请求:" + url + "请求参数有误不是map类型");
        }
        return url;
    }

}
