package com.chinaitop.depot.intelligent.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.Map;

/**
 * @author qingsong.han
 * @description: soap.xml soap调用类
 * @create 2020-04-19 13:35
 */
public class SoapConstant {
    private final static Logger log = LoggerFactory.getLogger(SoapConstant.class);
    // getSafetyShebei 在线设备url后缀
    private static final String getSafetyShebei = "/getSafetyShebei";
    // startFullMeasure 开始检测
    private static final String startFullMeasure = "/startFullMeasure";
    // 获取检测结果
    private static final String getMeasureFullWeight = "/getMeasureFullWeight";
    // 获取扫描进度
    private static final String getMeasureFullStatus = "/getMeasureFullStatus";
    // 设置数量检测粮情基础信息
    private static final String setBaseDATA = "/setBaseDATA";
    private static final String delBaseData = "/delBaseDATA"; // 删除设置

    /**
     * 效验在线设备
     *
     * @param value 检测设备编号
     * @param ws    webservice地址
     * @return xml字符串
     */
    public static boolean getSafetyShebei(String value, String ws) throws IOException {
        // 第一个参数: webservice地址加条件参数拼接
        return HttpUtil.doHttpGetByHttpClient(ws + getSafetyShebei + "?" + value, "text/xml;charset=utf8", "").contains(value + ":");
    }

    /**
     * 发送检测指令
     *
     * @param value
     * @param ws
     * @return
     * @throws IOException
     */
    public static boolean startFullMeasure(String value, String ws) throws IOException {
        // 第一个参数: webservice地址加条件参数拼接
        return HttpUtil.doHttpGetByHttpClient(ws + startFullMeasure + "?shebei=" + value, "text/xml;charset=utf8", "").contains("suc_");
    }

    /**
     * 获取检测结果
     *
     * @param value
     * @param ws
     * @return
     * @throws IOException
     */
    public static String getMeasureFullWeight(String value, String ws) throws IOException {
        // 第一个参数: webservice地址加条件参数拼接
        return HttpUtil.doHttpGetByHttpClient(ws + getMeasureFullWeight + "?shebei=" + value, "text/xml;charset=utf8", "");
    }

    /**
     * 获取扫描进度
     *
     * @param value
     * @param ws
     * @return
     * @throws IOException
     */
    public static String getMeasureFullStatus(String value, String ws) throws IOException {
        // 第一个参数: webservice地址加条件参数拼接
        return HttpUtil.doHttpGetByHttpClient(ws + getMeasureFullStatus + "?shebei=" + value, "text/xml;charset=utf8", "");
    }

    /**
     * 设置数量检测粮情基础信息
     *
     * @param mapValue 传入map参数与put进的参数倒叙
     * @param ws       基础配置url: http://127.0.0.1:20013/MyWebService.asmx
     * @throws IOException
     */
    public static String setBaseDATA(Map<String, Object> mapValue, String ws) throws IOException {
        // 请求参数: webservice地址+setBaseDATA+条件参数拼接
        String urlJoin = HttpUtil.getForObject(ws + setBaseDATA, mapValue);
        log.info("setBaseData请求值为: {}", urlJoin);
        return HttpUtil.doHttpGetByHttpClient(urlJoin, "text/xml;charset=utf8", "");
    }

    /**
     * 设置数量检测粮情基础信息
     *
     * @param value 传入map参数与put进的参数倒叙
     * @param ws       基础配置url: http://127.0.0.1:20013/MyWebService.asmx
     * @throws IOException
     */
    public static String delBaseData(String value, String ws) throws IOException {
        String getUrl = ws + delBaseData + "?key=" + value;
        log.info("delBaseData请求值为: {}", getUrl);
        // 请求参数: webservice地址
        return HttpUtil.doHttpGetByHttpClient(getUrl, "text/xml;charset=utf8", "");
    }

    // -------------------------以下20012老设备方法-----------------------
    // getSafetyShebei 在线设备url后缀
    private static final String getOnlineShebei = "/getOnlineShebei";
    // 扫描进度
    private static final String getMeasureStatus = "/getMeasureStatus?shebei=";
    // 开始检测
    private static final String getScanWeight = "/getScanWeight?shebei=";

    /**
     * 获取在线设备
     *
     * @param value 检测设备编号
     * @param ws    webservice地址
     * @return xml字符串
     */
    public static boolean getOnlineShebei(String value, String ws) throws IOException {
        // 第一个参数: webservice地址加条件参数拼接
        return HttpUtil.doHttpGetByHttpClient(ws + getOnlineShebei, "text/xml;charset=utf8", "").contains("suc_");
    }

    // 扫描进度
    public static boolean getMeasureStatus(String value, String ws) throws IOException {
        // 第一个参数: webservice地址加条件参数拼接
        return HttpUtil.doHttpGetByHttpClient(ws + getMeasureStatus + value, "text/xml;charset=utf8", "").contains("suc_");
    }

    // 获得测重结果
    public static boolean getScanWeight(String value, String ws) throws IOException {
        // 第一个参数: webservice地址加条件参数拼接
        return HttpUtil.doHttpGetByHttpClient(ws + getScanWeight + value, "text/xml;charset=utf8", "").contains("suc_");
    }

}