package com.chinaitop.depot.intelligent.utils;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMXMLBuilderFactory;

import java.io.ByteArrayInputStream;
import java.util.HashMap;
import java.util.Map;

/**
 * @author qingsong.han
 * @description: 解析soap
 * @create 2020-04-19 20:47
 */
public class SoapParseUtil {

    /**
     * soap解析
     *
     * @param soapXml soap.xml字符串
     * @param in      递归次数,获取body标签,传入最小值为1,三级父标签.其他情况传如0.
     * @return OMElement
     */
    public static OMElement soapParse(String soapXml, int in) {
        // 初始化body元素对象
        OMElement body = null;
        // 创建返回map对象
        Map<String, Object> map = new HashMap<>();
        // 利用axis2的OMElement,将xml数据转换为OMElement
        OMElement omElement = getDocumentOMElement(soapXml);
        if (in < 1) { //一级父标签
            body = getBody(omElement);
        }
        // 需要获取的值有三级以上的父标签,传入最小值1.
        for (int i = 0; i < in; i++) {
            // 递归获取xml标签body
            body = getBody(getBody(omElement));
        }
        return body;
    }

    /**
     * 递归方法
     *
     * @param omElement 获取body
     * @return OMElement
     */
    private static OMElement getBody(OMElement omElement) {
        return omElement.getFirstElement();
    }

    /**
     * 无父级标签直接调用
     * @param soapXml
     * @return
     */
    public static OMElement getDocumentOMElement(String soapXml){
        return OMXMLBuilderFactory.createOMBuilder(new ByteArrayInputStream(soapXml.getBytes()), "utf-8").getDocumentElement();
    }

}
