package com.chinaitop.depot.intelligent.ventilation.controller;

import com.chinaitop.depot.intelligent.ventilation.model.TAerationWarning;
import com.chinaitop.depot.intelligent.ventilation.service.AerationParameterService;
import com.chinaitop.depot.intelligent.ventilation.service.TAerationWarningService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.unissoft.model.ResponseEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value = "/intelligents/aerationRule")
@Api(value = "TAerationWarningController", description = "通风规则设置")
public class TAerationWarningController {

    @Autowired
    private TAerationWarningService tAerationWarningService;

   
    @RequestMapping(value = "/getList", method = RequestMethod.GET)
    @ApiOperation(value = "查询 通风规则列表", notes = "查询 通风规则列表，支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "库点编码", paramType = "query"),
            @ApiImplicitParam(name = "vcfcode", value = "仓库名称", paramType = "query")
    })
   public ResponseEntity<PageInfo<TAerationWarning>> getList(Integer pageNum, Integer pageSize, Integer orgId, String vCfCode) {
        List<TAerationWarning> list = null;
        try {
            if (null != pageNum && null != pageSize) {
                PageHelper.startPage(pageNum, pageSize);
            }
            list = tAerationWarningService.getList(orgId,vCfCode);
        } catch (Exception e) {
            e.printStackTrace();
            ResponseEntity.failed(e.getMessage());
        }
        PageInfo<TAerationWarning> pageInfo = new PageInfo<TAerationWarning>(list);
        return ResponseEntity.ok(pageInfo);
    }

    
    @RequestMapping(value = "/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "数据更新与添加", notes = "数据更新与添加")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "aerationRuleJson", value = "表数据json串", paramType = "form")
    })
    public ResponseEntity save(String aerationRuleJson) {
        try {
        	tAerationWarningService.save(aerationRuleJson);
        } catch (Exception e) {
            e.printStackTrace();
            ResponseEntity.failed(e.getMessage());
        }
        return ResponseEntity.ok();
    }
    
    @RequestMapping(value = "/getById", method = RequestMethod.POST)
    @ApiOperation(value = "根据id查询信息", notes = "根据id查询信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "query")
    })
    public ResponseEntity<TAerationWarning> getById(String id) {
    	TAerationWarning tAerationWarning = null;
        if (StringUtils.isNotBlank(id)) {
        	tAerationWarning = tAerationWarningService.getById(id);
        } else {
            return ResponseEntity.failed("ID数据有误!");
        }
        return ResponseEntity.ok(tAerationWarning);
    }
    
    @RequestMapping(value = "/deleteById", method = RequestMethod.POST)
    @ApiOperation(value = "根据粮情id删除通风规则信息", notes = "根据id删除通风规则信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "通风规则id", paramType = "query"),
            @ApiImplicitParam(name = "aerationParamJson", value = "通风参数json字符串", paramType = "query")
    })
    public ResponseEntity deleteById(String id) {
        if (StringUtils.isNotBlank(id)) {
            int count = tAerationWarningService.deleteById(id);
            if (count > 0) {
                return ResponseEntity.ok();
            } else {
                return ResponseEntity.failed("删除失败!");
            }
        }
        return ResponseEntity.failed("ID数据有误!");
    }
    
    
}
